"""
quizlib type definitions
"""

from enum import Enum, auto


class QuestionType(Enum):
    """Question type enumeration."""

    MULTIPLE_CHOICE = auto()
    ESSAY = auto()
    TRUE_FALSE = auto()
    CLOZE = auto()


# Backwards-compatible alias
Qtype = QuestionType

# Type alias for MC answers: (is_correct: bool, answer_text: str)
type MCAnswer = tuple[bool, str]
