"""
quizlib base protocol for renderers
"""

from typing import TYPE_CHECKING, Protocol

if TYPE_CHECKING:
    from quizlib.quiz import Quiz


class Renderer(Protocol):
    """Protocol that all renderers must implement."""

    def render(self, quiz: "Quiz") -> str:
        """Render a quiz to the target format."""
        ...
