"""
quizlib base protocol for questions
"""

from typing import Protocol, runtime_checkable

from quizlib.types import QuestionType


@runtime_checkable
class Question(Protocol):
    """Protocol that all question types must implement."""

    name: str
    text: str
    typ: QuestionType
