"""
quizlib question type implementations
"""

from quizlib.questions.base import Question
from quizlib.questions.cloze import CZ, Cloze
from quizlib.questions.essay import ES, Essay
from quizlib.questions.multiple_choice import MC, MultipleChoice
from quizlib.questions.true_false import TF, TrueFalse

__all__ = [
    # Protocol
    "Question",
    # Full names
    "Essay",
    "MultipleChoice",
    "TrueFalse",
    "Cloze",
    # Short aliases
    "ES",
    "MC",
    "TF",
    "CZ",
]
