"""
quizlib - Generate quizzes from Python.

Example usage:

>>> from quizlib import Quiz, Essay, MultipleChoice, TrueFalse, Cloze
>>> from quizlib.renderers import MoodleXMLRenderer
>>>
>>> quiz = Quiz(category="MyQuiz", info="A sample quiz")
>>> quiz.add(Essay(name="q1", defgrade=5, text="Explain X..."))
>>> quiz.add(MultipleChoice(
>>>     name="q2",
>>>     text="Which is correct?",
>>>     answers=[(1, "Correct"), (0, "Wrong A"), (0, "Wrong B")],
>>> ))
>>>
>>> renderer = MoodleXMLRenderer()
>>> print(renderer.render(quiz))
"""

from quizlib.questions import (
    CZ,
    ES,
    MC,
    TF,
    Cloze,
    Essay,
    MultipleChoice,
    Question,
    TrueFalse,
)
from quizlib.quiz import Quiz
from quizlib.types import QuestionType, Qtype

__all__ = [
    # Core
    "Quiz",
    "Question",
    "QuestionType",
    "Qtype",
    # Question types (full names)
    "Essay",
    "MultipleChoice",
    "TrueFalse",
    "Cloze",
    # Question types (aliases)
    "ES",
    "MC",
    "TF",
    "CZ",
]
